#!perl -w

package DE_EPAGES::Core::t::GitHub;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::GitHub;
use DE_EPAGES::Test::Mock::Mockify;

use Test::More tests => 20;
use Test::Exception;

use strict;

sub test {
    my $self = shift;
    $self->test_CreatePullRequest_Die_1();
    $self->test_CreatePullRequest_Die_2();
    $self->test_CreatePullRequest_Success();
    $self->test_MergePullRequest_Die();
    $self->test_MergePullRequest_Success();
    $self->test_IsMergeable_Die();
    $self->test_IsMergeable_True();
    $self->test_IsMergeable_False();
    $self->test_IsMerged_Die();
    $self->test_IsMerged_True();
    $self->test_IsMerged_False();
    done_testing();
}

sub test_CreatePullRequest_Die_1 {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyTitle = "Test title.";
    my $DummyFeatureBranch = "issue/featureBranch";
    my $DummyTargetBranch = "issue/targetBranch";

    my $GitHub = $self->_createGitHub($self->_createJSONRestClientSuccess());

    # Dies because of missig parameters.
    dies_ok( sub { $GitHub->CreatePullRequest() }, "$SubTestName - Dies without title.");
    dies_ok( sub { $GitHub->CreatePullRequest($DummyTitle) }, "$SubTestName - Dies without feature branch.");
    dies_ok( sub { $GitHub->CreatePullRequest($DummyTitle, $DummyFeatureBranch) }, "$SubTestName - Dies without branch.");
    dies_ok( sub { $GitHub->CreatePullRequest($DummyTitle, $DummyFeatureBranch, $DummyTargetBranch) }, "$SubTestName - Dies without repo.");
}

sub test_CreatePullRequest_Die_2 {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyTitle = "Test title.";
    my $DummyFeatureBranch = "issue/featureBranch";
    my $DummyTargetBranch = "issue/branch";
    my $DummyRepo = "origin";
    my $DummyComment = "Test comment.";

    my $GitHub = $self->_createGitHub($self->_createJSONRestClientFailure());

    # Dies because of missing pull request ID.
    dies_ok( sub { $GitHub->CreatePullRequest($DummyTitle, $DummyFeatureBranch, $DummyTargetBranch, $DummyRepo) },
        "$SubTestName - Dies because of missing return of the pull request id.");
    dies_ok( sub { $GitHub->CreatePullRequest($DummyTitle, $DummyFeatureBranch, $DummyTargetBranch, $DummyRepo, $DummyComment) },
        "$SubTestName - Dies because of missing return of the pull request id.");
}

sub test_CreatePullRequest_Success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyTitle = "Test title.";
    my $DummyFeatureBranch = "issue/featureBranch";
    my $DummyBranch = "issue/branch";
    my $DummyRepo = "origin";
    my $DummyComment = "Test comment.";

    my $GitHub = $self->_createGitHub($self->_createJSONRestClientSuccess());

    my $pullRequestNumber = $GitHub->CreatePullRequest($DummyTitle, $DummyFeatureBranch, $DummyBranch, $DummyRepo);
    is($pullRequestNumber, 12345, "$SubTestName - Successful returned pull request id.");

    $pullRequestNumber = 0;

    $pullRequestNumber = $GitHub->CreatePullRequest($DummyTitle, $DummyFeatureBranch, $DummyBranch, $DummyRepo, $DummyComment);
    is($pullRequestNumber, 12345, "$SubTestName - Successful returned pull request id.");
}

sub test_MergePullRequest_Die {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyCommitMessage = "Some dummy commit message.";
    my $DummyPRID = "12345";

    my $GitHub = $self->_createGitHub($self->_createJSONRestClientSuccess());

    dies_ok( sub { $GitHub->MergePullRequest() }, "$SubTestName - Dies without parameters.");
    dies_ok( sub { $GitHub->MergePullRequest($DummyCommitMessage) }, "$SubTestName - Dies without dummy pull request ID.");
    dies_ok( sub { $GitHub->MergePullRequest($DummyCommitMessage, $DummyPRID) }, "$SubTestName - Dies without repo.");
}

sub test_MergePullRequest_Success {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyCommitMessage = "Some dummy commit message.";
    my $DummyPRID = "12345";
    my $DummyRepo = "origin";

    my $GitHub = $self->_createGitHub($self->_createJSONRestClientSuccess());

    lives_ok(sub { $GitHub->MergePullRequest($DummyCommitMessage, $DummyPRID, $DummyRepo) },
        "$SubTestName - Successful with commit message, pull request ID and repo.");
}

sub test_IsMergeable_Die {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyPRID = "12345";

    my $GitHub = $self->_createGitHub($self->_createJSONRestClientSuccess());

    dies_ok( sub { $GitHub->IsMergable() }, "$SubTestName - Dies without parameters.");
    dies_ok( sub { $GitHub->IsMergable($DummyPRID) }, "$SubTestName - Dies without repo.");
}

sub test_IsMergeable_True {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyPRID = "12345";
    my $DummyRepo = "origin";

    my $GitHub = $self->_createGitHub($self->_createJSONRestClientMergeable());
    my $WasSuccessful = $GitHub->IsMergeable($DummyPRID, $DummyRepo);

    is($WasSuccessful, 1, "$SubTestName - Successful returning true.");
}

sub test_IsMergeable_False {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyPRID = "12345";
    my $DummyRepo = "origin";

    my $GitHub = $self->_createGitHub($self->_createJSONRestClientMerged());
    my $WasSuccessful = $GitHub->IsMergeable($DummyPRID, $DummyRepo);

    is($WasSuccessful, 0, "$SubTestName - Successful returning false.");
}

sub test_IsMerged_Die {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyPRID = "12345";

    my $GitHub = $self->_createGitHub($self->_createJSONRestClientSuccess());

    dies_ok( sub { $GitHub->IsMerged() }, "$SubTestName - Dies without parameters.");
    dies_ok( sub { $GitHub->IsMerged($DummyPRID) }, "$SubTestName - Dies without repo.");
}

sub test_IsMerged_True {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyPRID = "12345";
    my $DummyRepo = "origin";

    my $GitHub = $self->_createGitHub($self->_createJSONRestClientMerged());
    my $WasSuccessful = $GitHub->IsMerged($DummyPRID, $DummyRepo);

    is($WasSuccessful, 1, "$SubTestName - Successful returning true.");
}

sub test_IsMerged_False {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $DummyPRID = "12345";
    my $DummyRepo = "origin";

    my $GitHub = $self->_createGitHub($self->_createJSONRestClientMergeable());
    my $WasSuccessful = $GitHub->IsMerged($DummyPRID, $DummyRepo);

    is($WasSuccessful, 0, "$SubTestName - Successful returning false.");
}

sub _createGitHub {
    my $self = shift;

    my ($JSONRestClient) = @_;

    my $aParameterList = ['jDoe', 'password'];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                    'DE_EPAGES::Core::API::GitHub',
                    $aParameterList
                );

    $Mockify->addMockWithReturnValue('_createJSONRestClient', $JSONRestClient);
    $Mockify->addMockWithReturnValueAndParameterCheck('_createJSONRestClientWithBaseUrl', $JSONRestClient, ['string']);

    return $Mockify->getMockObject();
}

sub _createJSONRestClientSuccess {
    my $self = shift;

    my $aParameterList = ['https://api.github.com', {'' => 'fancyToken'}];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                    'DE_EPAGES::WebService::API::JSONRestClient',
                    $aParameterList
                );
    my $JSONData = { 'number' => 12345 };

    my $FakePutResponse = { 'sha' => '6dcb09b5b57875f334f61aebed695e2e4193db5e' };
    $Mockify->addMockWithReturnValueAndParameterCheck('put', $FakePutResponse, ['string', 'hashref', 'int']);
    $Mockify->addMockWithReturnValueAndParameterCheck('post', $JSONData, ['hashref']);

    return $Mockify->getMockObject();
}

sub _createJSONRestClientFailure {
    my $self = shift;

    my $aParameterList = ['https://api.github.com', {'' => 'fancyToken'}];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                    'DE_EPAGES::WebService::API::JSONRestClient',
                    $aParameterList
                );

    my $JSONData = { 'message' => '' };

    $Mockify->addMockWithReturnValueAndParameterCheck('put', '', ['string', 'hashref']);
    $Mockify->addMockWithReturnValueAndParameterCheck('post', $JSONData, ['hashref']);

    return $Mockify->getMockObject();
}

sub _createJSONRestClientMergeable {
    my $self = shift;

    my $aParameterList = ['https://api.github.com', {'' => 'fancyToken'}];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                    'DE_EPAGES::WebService::API::JSONRestClient',
                    $aParameterList
                );
    my $JSONData = { 'mergeable' => 1 };
    $Mockify->addMockWithReturnValueAndParameterCheck('get', $JSONData, ['string']);

    return $Mockify->getMockObject();
}

sub _createJSONRestClientMerged {
    my $self = shift;

    my $aParameterList = ['https://api.github.com', {'' => 'fancyToken'}];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                    'DE_EPAGES::WebService::API::JSONRestClient',
                    $aParameterList
                );

    my $JSONData = { 'merged' => 1 };
    $Mockify->addMockWithReturnValueAndParameterCheck('get', $JSONData, ['string']);

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();