#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::Tools
# state        public
# description  Unit test for Tools.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::Tools;
use base DE_EPAGES::Core::API::Test;
use strict;

use DE_EPAGES::DHLIntraship::API::Constants qw ( $MAX_ATTRIBUTE_LENGTH );
use DE_EPAGES::DHLIntraship::API::Tools;
use DE_EPAGES::Test::Mock::Mockify;
use Readonly;
use Test::Exception;
use Test::More;

sub test {
    my $self = shift;
    $self->_test_truncateAttribute();
    $self->_test_truncateAttribute_FieldNameDoesNotExist();
    $self->_test_truncateAttribute_MissingFieldName();
    $self->_test_truncateAttribute_MissingParameters();
    done_testing();
}

sub _test_truncateAttribute {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTools        = $self->_createTools();
    my $CompanyName        = 'Xavizonmycompanynamehasmorethanthirtycharacters';
    my $TruncatedAttribute = $MockedTools->truncateAttribute( $CompanyName, 'Company' );

    ok( length($TruncatedAttribute) <= $MAX_ATTRIBUTE_LENGTH->{'Company'}, "$SubTestName - Company name has <= 30 characters." );
}

sub _test_truncateAttribute_FieldNameDoesNotExist {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTools        = $self->_createTools();
    my $MadeUpAttribute    = 'Thisinventedattributehasmorethanthirtycharacters';
    my $TruncatedAttribute = $MockedTools->truncateAttribute( $MadeUpAttribute, 'EasterEgg' );

    ok( $TruncatedAttribute eq $MadeUpAttribute, "$SubTestName - Attribute has not been modified." );
}

sub _test_truncateAttribute_MissingFieldName {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTools        = $self->_createTools();
    my $MadeUpAttribute    = 'Thisinventedattributehasmorethanthirtycharacters';
    my $TruncatedAttribute = $MockedTools->truncateAttribute($MadeUpAttribute);

    ok( $TruncatedAttribute eq $MadeUpAttribute, "$SubTestName - Attribute has not been modified." );
}

sub _test_truncateAttribute_MissingParameters {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedTools = $self->_createTools();
    lives_ok( sub { $MockedTools->truncateAttribute() }, "$SubTestName - Method does not die." );
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createTools {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::DHLIntraship::API::Tools');

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();
1;
