#========================================================================================
# package      DE_EPAGES::Order::t::unit::SaleSummary
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Order::API::SaleSummary
#========================================================================================
package DE_EPAGES::Order::t::unit::SaleSummary;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Order::API::Sale::SaleSummary;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::Exception;
use Test::More;

use strict;

## no critic (ValuesAndExpressions::ProhibitMagicNumbers)



sub test {
    my $self = shift;

    $self->test_new_invalidShop();
    $self->test_get();

    done_testing();
    return;
}



sub test_new_invalidShop {
    my $self = shift;

    my $InvalidShop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::Object', {} );
    throws_ok { DE_EPAGES::Order::API::Sale::SaleSummary->new($InvalidShop) } qr/WRONG_CLASS/, 'test if error is thrown when object is not a shop';

    return;
}



sub test_get {
    my $self = shift;

    my $Shop          = $self->_createMockedShop();
    my $SaleSummary = $self->_createSaleSummary( $Shop );
    my $DateFrom = 'start-date';
    my $DateTo   = 'end-date';
    my $hSummary = $SaleSummary->get( $DateFrom, $DateTo );

    my $hExpectedSummary = {
        'salesPerCurrency' => [
            {
                'currency'     => 'EUR',
                'totalNetRevenue' => 123.45,
                'totalGrossRevenue' => 150.55,
                'totalOrders' => 4,
            },
            {
                'currency'     => 'GBP',
                'totalNetRevenue' => 3.14,
                'totalGrossRevenue' => 3.99,
                'totalOrders' => 3,
            },
        ]
    };

    is_deeply( $hSummary, $hExpectedSummary, 'SaleSummary multi currency and single revenue' );

    return;
}



sub _createMockedShop {
    my $self = shift;
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', {
        'ID' => 123,
    });
}



sub _createSaleSummary {
    my $self = shift;
    my ( $Shop ) = @_;

    my $Mockery = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Order::API::Sale::SaleSummary', [$Shop]);

    my $aResult = [
        [
            123.45,
            3,
            'EUR',
            150.55,
            4
        ],
        [
            3.14,
            2,
            'GBP',
            3.99,
            3
        ]
    ];

    $Mockery->addMockWithReturnValueAndParameterCheck('_DAL_GetSalesVolumeOverAllCurrencies', $aResult, [{
        'string' => '123'
    },
    {
        'string' => 'start-date'
    },
    {
        'string' => 'end-date'
    }]);

    return $Mockery->getMockObject();
}



__PACKAGE__->runScript();

1;